@@ -425,6 +425,20 @@ set_xref(Xref) :-
     current_prolog_flag(xref, Xref),
     set_prolog_flag(xref, true).
 
+:- meta_predicate
+    with_xref(0).
+
+with_xref(Goal) :-
+    current_prolog_flag(xref, Xref),
+    (   Xref == true
+    ->  call(Goal)
+    ;   setup_call_cleanup(
+            set_prolog_flag(xref, true),
+            Goal,
+            set_prolog_flag(xref, Xref))
+    ).
+
+
 %!  set_initial_mode(+Stream, +Options) is det.
 %
 %   Set  the  initial  mode  for  processing    this   file  in  the
@@ -1243,7 +1257,7 @@ xref_meta_src(Head, Called, _) :-
     arg(1, Head, G),
     Called = [G+Extra].
 xref_meta_src(Head, Called, _) :-
-    predicate_property('$xref_tmp':Head, meta_predicate(Meta)),
+    with_xref(predicate_property('$xref_tmp':Head, meta_predicate(Meta))),
     !,
     Meta =.. [_|Args],
     meta_args(Args, 1, Head, Called).
@@ -1470,7 +1484,7 @@ hook(sandbox:safe_meta_predicate(_)).
 hook(sandbox:safe_meta(_,_)).
 hook(sandbox:safe_global_variable(_)).
 hook(sandbox:safe_directive(_)).
-
+hook(sandbox:safe_prolog_flag(_,_)).
 
 %!  arith_callable(+Spec, -Callable)
 %
