@@ -81,9 +81,8 @@ please visit https://www.swi-prolog.org/pldoc/package/plunit.
 		 *    CONDITIONAL COMPILATION   *
 		 *******************************/
 
-swi     :- catch(current_prolog_flag(dialect, swi), _, fail), !.
-swi     :- catch(current_prolog_flag(dialect, yap), _, fail).
-sicstus :- catch(current_prolog_flag(system_type, _), _, fail).
+swi     :- catch(current_prolog_flag(dialect, swi),     _, fail).
+sicstus :- catch(current_prolog_flag(dialect, sicstus), _, fail).
 
 throw_error(Error_term,Impldef) :-
     throw(error(Error_term,context(Impldef,_))).
@@ -134,15 +133,25 @@ goal_expansion(current_module(Module,File),
 :- initialization init_flags.
 
 init_flags :-
-    (   global_test_option(Option, _Value, _Type, Default),
+    (   global_test_option(Option, _Value, Type, Default),
 	Default \== (-),
 	Option =.. [Name,_],
 	atom_concat(plunit_, Name, Flag),
-	create_prolog_flag(Flag, Default, [keep(true)]),
+        flag_type(Type, FlagType),
+	create_prolog_flag(Flag, Default, [type(FlagType), keep(true)]),
 	fail
     ;   true
     ).
 
+flag_type(boolean, FlagType) => FlagType = boolean.
+flag_type(Type, FlagType), Type = oneof(Atoms), maplist(atom, Atoms) =>
+    FlagType = Type.
+flag_type(oneof(_), FlagType) => FlagType = term.
+flag_type(positive_integer, FlagType) => FlagType = integer.
+flag_type(number, FlagType) => FlagType = float.
+
+
+
 %!  set_test_options(+Options)
 %
 %   Specifies how to deal with test suites.  Defined options are:
@@ -573,7 +582,8 @@ test_set_option(timeout(Seconds)) :-
        reify_tmo(0, -, +),
        reify(0, -),
        capture_output(0,-),
-       capture_output(0,-,+).
+       capture_output(0,-,+),
+       got_messages(0,-).
 
 %!  reify_tmo(:Goal, -Result, +Options) is det.
 
@@ -605,20 +615,55 @@ reify(Goal, Result) :-
     ;   Result = false
     ).
 
+%!  capture_output(:Goal, -Output) is semidet.
+%!  capture_output(:Goal, -Output, +Options) is semidet.
+%
+%   @arg Output is a pair `Msgs-String`, where  `Msgs` is a boolean that
+%   is true if there were messages that   require a non-zero exit status
+%   and Output contains the output as a string.
+
 capture_output(Goal, Output) :-
     current_test_flag(output, OutputMode),
     capture_output(Goal, Output, [output(OutputMode)]).
 
-capture_output(Goal, Output, Options) :-
+capture_output(Goal, Msgs-Output, Options) :-
     option(output(How), Options, always),
     (   How == always
-    ->  call(Goal)
-    ;   with_output_to(string(Output), Goal,
+    ->  call(Goal),
+        Msgs = false                % irrelavant
+    ;   with_output_to(string(Output), got_messages(Goal, Msgs),
                        [ capture([user_output, user_error]),
                          color(true)
                        ])
     ).
 
+%!  got_messages(:Goal, -Result)
+
+got_messages(Goal, Result) :-
+    (   current_prolog_flag(on_warning, status)
+    ;   current_prolog_flag(on_error, status)
+    ), !,
+    nb_delete(plunit_got_message),
+    setup_call_cleanup(
+        asserta(( user:thread_message_hook(_Term, Kind, _Lines) :-
+                      got_message(Kind), fail), Ref),
+        Goal,
+        erase(Ref)),
+    (   nb_current(plunit_got_message, true)
+    ->  Result = true
+    ;   Result = false
+    ).
+got_messages(Goal, false) :-
+    call(Goal).
+
+:- public got_message/1.
+got_message(warning) :-
+    current_prolog_flag(on_warning, status), !,
+    nb_setval(plunit_got_message, true).
+got_message(error) :-
+    current_prolog_flag(on_error, status), !,
+    nb_setval(plunit_got_message, true).
+
 
 		 /*******************************
 		 *        RUNNING TOPLEVEL      *
@@ -1341,6 +1386,10 @@ success(Unit, Name, Progress, Line, _, Time, Output, Options) :-
     failed_assertion(Unit, Name, Line, _,Progress,_,_),
     !,
     failure(Unit, Name, Progress, Line, assertion, Time, Output, Options).
+success(Unit, Name, Progress, Line, _, Time, Output, Options) :-
+    Output = true-_,
+    !,
+    failure(Unit, Name, Progress, Line, message, Time, Output, Options).
 success(Unit, Name, Progress, Line, Det, Time, _Output, Options) :-
     assert(passed(Unit, Name, Line, Det, Time)),
     (   (   Det == true
@@ -2042,9 +2091,13 @@ unqualify(M:Goal, _, Goal) :-
     !.
 unqualify(Goal, _, Goal).
 
+test_output(Msgs-String) -->
+    { nonvar(Msgs) },
+    !,
+    test_output(String).
 test_output("") --> [].
 test_output(Output) -->
-    [ ansi(code, '~s', [Output]) ].
+    [ ansi(code, '~N~s', [Output]) ].
 
 :- endif.
 					% Setup/condition errors
@@ -2186,6 +2239,9 @@ failure(throw(Error)) -->
     },
     [ 'received error: ~w'-[Message] ].
 :- endif.
+failure(message) -->
+    !,
+    [ 'Generated unexpected warning or error'-[] ].
 failure(Why) -->
     [ '~p'-[Why] ].
 
@@ -2381,9 +2437,13 @@ hide_progress(forall(_,_)).
 hide_progress(failed).
 hide_progress(timeout(_)).
 
+print_test_output(Error, _Msgs-Output) =>
+    print_test_output(Error, Output).
 print_test_output(_, "") => true.
 print_test_output(assertion, Output) =>
     print_message(debug, plunit(test_output(error, Output))).
+print_test_output(message, Output) =>
+    print_message(debug, plunit(test_output(error, Output))).
 print_test_output(_, Output) =>
     print_message(debug, plunit(test_output(informational, Output))).
 
@@ -2488,7 +2548,10 @@ tty_header_line(Width) :-
 
 :- if(current_predicate(tty_get_capability/3)).
 tty_clear_to_eol(S) :-
-    tty_get_capability(ce, string, S),
+    getenv('TERM', _),
+    catch(tty_get_capability(ce, string, S),
+          error(_,_),
+          fail),
     !.
 :- endif.
 tty_clear_to_eol('\e[K').
