@@ -296,6 +296,11 @@ PL_EXPORT_DATA(int)		Slinesize;		/* Sgets() linesize */
 PL_EXPORT_DATA(IOSTREAM)	S__iob[3];		/* Libs standard streams */
 #endif
 
+/* WARNING: Sinput, Soutput, Serror use the OS's files directly.
+            If you wish to use Prolog's streams, use Suser_input,
+            Scurrent_output, etc. in SWI-Prolog.h
+*/
+
 #define Sinput  (&S__iob[0])		/* Stream Sinput */
 #define Soutput (&S__iob[1])		/* Stream Soutput */
 #define Serror  (&S__iob[2])		/* Stream Serror */
@@ -471,6 +476,9 @@ PL_EXPORT(IOSTREAM *)	Sopen_iri_or_file(const char *path, const char *how);
 PL_EXPORT(IOSTREAM *)	Sfdopen(int fd, const char *type);
 PL_EXPORT(int)		Sfileno(IOSTREAM *s);
 #ifdef __WINDOWS__
+PL_EXPORT(int)		Swin_open_osfhandle(HANDLE h, int flags);
+PL_EXPORT(IOSTREAM *)	Swin_open_handle(HANDLE h, const char *mode);
+PL_EXPORT(HANDLE)	Swinhandle(IOSTREAM *s);
 #if defined(_WINSOCKAPI_) || defined(NEEDS_SWINSOCK) /* have SOCKET */
 PL_EXPORT(SOCKET)	Swinsock(IOSTREAM *s);
 #endif
