@@ -490,7 +490,6 @@ struct PL_local_data
   int		break_level;		/* current break level */
   Stack		outofstack;		/* thread is out of stack */
   int		trim_stack_requested;	/* perform a trim-stack */
-  int		in_arithmetic;		/* doing arithmetic */
   int		in_print_message;	/* Inside printMessage() */
   void *	glob_info;		/* pl-glob.c */
   IOENC		encoding;		/* default I/O encoding */
@@ -822,8 +821,8 @@ struct PL_local_data
 
 #ifdef O_COVERAGE
   struct
-  { Table	table;			/* Table for recording data */
-    int		active;			/* We are recording */
+  { struct coverage *data;		/* Data for the coverage collection */
+    int		     active;		/* This thread is recording */
   } coverage;
 #endif
 
