@@ -36,6 +36,7 @@
 */
 
 #include "pl-incl.h"
+#include "os/pl-cstack.h"
 
 #ifndef PL_THREAD_H_DEFINED
 #define PL_THREAD_H_DEFINED
@@ -113,8 +114,7 @@ typedef struct _PL_thread_info_t
   unsigned	    c_stack_low   : 1;	/* TRUE: Signalled low C stack */
   unsigned	    joined_by_creator : 1;
   int		    joining_by;		/* TID of joining thread */
-  void		   *c_stack_base;	/* Base of the (C-) stack */
-  size_t	    c_stack_size;	/* system (C-) stack */
+  c_stack_info	   *c_stack;
   size_t	    stack_limit;	/* Stack sizes */
   size_t	    table_space;	/* Max size for local tables */
   rc_cancel	    (*cancel)(int id);	/* cancel function */
@@ -157,7 +157,7 @@ typedef struct thread_handle
   atom_t		symbol;		/* associated symbol */
   atom_t		alias;		/* alias name of the thread */
   int			engine_id;	/* numeric engine id */
-  int			flags;		/* symbol flags */
+  unsigned int		flags;		/* symbol flags */
   struct
   { qid_t query;			/* Query handle */
     term_t argv;			/* Arguments */
@@ -391,7 +391,7 @@ typedef DWORD	TLD_KEY;
 #define TLD_alloc(p)	(*(p) = TlsAlloc())
 #define TLD_get(p)	TlsGetValue((p))
 #define TLD_set(p, v)	TlsSetValue((p), (v))
-#define TLD_free(p)	TlsFree(p);
+#define TLD_free(p)	TlsFree(*p);
 
 #else
 typedef pthread_key_t TLD_KEY;
